# Generated by Django 3.1.1 on 2021-01-06 08:03

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0012_index_additions'),
        ('aastatistics', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='zKillStatsFilter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('kill_count', models.IntegerField(default=0)),
                ('months', models.IntegerField(default=1)),
            ],
            options={
                'verbose_name': 'Smart Filter: zKill: Kills in Period',
                'verbose_name_plural': 'Smart Filter: zKill: Kills in Period',
            },
        ),
        migrations.AlterField(
            model_name='statscharacter',
            name='character',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='character_stats', to='eveonline.evecharacter'),
        ),
        migrations.AlterField(
            model_name='statscharacter',
            name='last_update',
            field=models.DateTimeField(default=datetime.datetime(2019, 12, 28, 8, 3, 21, 524661)),
        ),
        migrations.AlterField(
            model_name='zkillmonth',
            name='last_update',
            field=models.DateTimeField(default=datetime.datetime(2019, 12, 28, 8, 3, 21, 525255)),
        ),
    ]
 


from django.db.models import Sum
from django.db.models.functions import Coalesce
from dateutil.relativedelta import relativedelta

import datetime

import logging

logger = logging.getLogger(__name__)


def check_kills_in_account(user, months, kills):
    from .models import StatsCharacter, zKillMonth
    """
    Check Account for x Kills for x months
 
    Parameters
    ==========
    user (model)
    Months (int)
    Kills (int)

    Returns
    ======
    Returns True/False
    """
    try:
        logger.debug("Checking kills for {0}, in the last {1} months, {2} kill threshold.".format(user.profile.main_character.character_name, months, kills))

        now = datetime.datetime.now()
        character_list = user.character_ownerships.all().select_related('character', 'character__character_stats')
        if months == 12:
            kill_count = 0
            for c in character_list:
                try:
                    kill_count += c.character.character_stats.zk_12m
                except Exception as e:
                    logger.error(e)
                    pass       
        elif months == 6:
            kill_count = 0
            for c in character_list:
                try:
                    kill_count += c.character.character_stats.zk_6m
                except Exception as e:
                    logger.error(e)
                    pass
        elif months == 3:
            kill_count = 0
            for c in character_list:
                try:
                    kill_count += c.character.character_stats.zk_3m
                except Exception as e:
                    logger.error(e)
                    pass
        else:
            character_ids = set(character_list.values_list('character__character_id', flat=True))

            dt = now - relativedelta(months=months)
            month_ago = dt.month
            year_ago = dt.year

            characters = StatsCharacter.objects.filter(character__character_id__in=character_ids)

            qs = zKillMonth.objects.filter(char__in=characters)

            qs_kills = qs.filter(year__gte=year_ago, month__gte=month_ago)
            if year_ago < now.year:
                qs_kills = qs_kills | qs.filter(year=now.year)

            kill_count = qs_kills.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        # logger.debug(kill_count)
        if kill_count >= kills:
            return True
        else:
            return False
    except:
        return False



# Generated by AaronKable on 2021-07-15 08:13
from django.db import migrations


def gen_date_str_fwd(apps, schema_editor):
    MyModel = apps.get_model('aastatistics', 'zKillMonth')
    for row in MyModel.objects.all():
        row.date_str = (str(row.year)+str(row.month).zfill(2))
        row.save(update_fields=['date_str'])


class Migration(migrations.Migration):

    dependencies = [
        ('aastatistics', '0003_auto_20210715_1031'),
    ]

    operations = [
        migrations.RunPython(gen_date_str_fwd, reverse_code=migrations.RunPython.noop),
    ]



from allianceauth.services.hooks import UrlHook
from allianceauth import hooks
from .models import zKillStatsFilter
from . import urls


@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'aastatistics', r'^aastatistics/')

@hooks.register("secure_group_filters")
def filters():
    return [zKillStatsFilter]


from django.db import models
from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import User
from allianceauth.authentication.models import CharacterOwnership
from django.db.models import Max, Sum, Q
from . import filters as smart_filters
from collections import defaultdict
from dateutil.relativedelta import relativedelta

import datetime


class StatsCharacter(models.Model):
    character = models.OneToOneField(EveCharacter, on_delete=models.CASCADE, related_name='character_stats')
    isk_destroyed = models.BigIntegerField(default=0)
    isk_lost = models.BigIntegerField(default=0)
    all_time_sum = models.IntegerField(default=0)
    gang_ratio = models.IntegerField(default=0)
    ships_destroyed = models.IntegerField(default=0)
    ships_lost = models.IntegerField(default=0)
    solo_destroyed = models.IntegerField(default=0)
    solo_lost = models.IntegerField(default=0)
    active_pvp_kills = models.IntegerField(default=0)
    last_kill = models.DateTimeField(null=True, default=None)
    last_update = models.DateTimeField(default=datetime.datetime.min)

    zk_12m = models.IntegerField(default=0)
    zk_6m = models.IntegerField(default=0)
    zk_3m = models.IntegerField(default=0)

    def __str__(self):
        return self.character.character_name


class zKillMonth(models.Model):
    char = models.ForeignKey(StatsCharacter, on_delete=models.CASCADE)
    year = models.IntegerField(default=0)
    month = models.IntegerField(default=0)
    date_str = models.CharField(max_length=6)
    ships_destroyed = models.IntegerField(default=0)
    ships_lost = models.IntegerField(default=0)
    isk_destroyed = models.BigIntegerField(default=0)
    isk_lost = models.BigIntegerField(default=0)
    last_update = models.DateTimeField(default=datetime.datetime.min)

    def __str__(self):
        return self.char.character.character_name

    class Meta:
        indexes = [
            models.Index(fields=['date_str']),
            models.Index(fields=['year']),
            models.Index(fields=['month']),
        ]


class FilterBase(models.Model):

    name = models.CharField(max_length=500)
    description = models.CharField(max_length=500)

    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.name}: {self.description}"

    def process_filter(self, user: User):
        raise NotImplementedError("Please Create a filter!")


class zKillStatsFilter(FilterBase):
    class Meta:
        verbose_name = "Smart Filter: zKill: Kills in Period"
        verbose_name_plural = verbose_name

    kill_count = models.IntegerField(default=0)
    months = models.IntegerField(default=1)

    def process_filter(self, user: User):
        return smart_filters.check_kills_in_account(
            user, self.months, self.kill_count
        )


    def audit_filter(self, users):
        now = datetime.datetime.now()
        dt = now - relativedelta(months=self.months)
        date_str = str(dt.year)+str(dt.month).zfill(2)

        _cos = CharacterOwnership.objects.filter(user__in=users).annotate(
            kills=Sum('character__character_stats__zkillmonth__ships_destroyed',
                      filter=Q(character__character_stats__zkillmonth__date_str__gte=date_str)
                )
            ).values("user_id", 'character_id', "kills")
        
        co = {}
        for c in _cos:
            u = c.get('user_id')
            if u not in co:
                co[u] = 0
            if c.get('kills'):
                co[u] += c.get('kills')

        output = defaultdict(lambda: {"message": "", "check": False})
        for u, c in co.items():
            if c:
                if c >= self.kill_count:
                    check = True
                else:
                    check = False
                msg = str(c) + " Kills"
            else:
                check = False
                msg = "0 Kill Data"

            output[u] = {"message": msg, "check": check}
        return output



# Generated by Django 3.2.5 on 2021-07-15 10:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aastatistics', '0005_alter_zkillmonth_date_str'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='zkillmonth',
            index=models.Index(fields=['date_str'], name='aastatistic_date_st_cf872c_idx'),
        ),
        migrations.AddIndex(
            model_name='zkillmonth',
            index=models.Index(fields=['year'], name='aastatistic_year_da44d7_idx'),
        ),
        migrations.AddIndex(
            model_name='zkillmonth',
            index=models.Index(fields=['month'], name='aastatistic_month_52ce12_idx'),
        ),
    ]



# Generated by Django 3.1.4 on 2021-01-03 10:38

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0012_index_additions'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatsCharacter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('isk_destroyed', models.BigIntegerField(default=0)),
                ('isk_lost', models.BigIntegerField(default=0)),
                ('all_time_sum', models.IntegerField(default=0)),
                ('gang_ratio', models.IntegerField(default=0)),
                ('ships_destroyed', models.IntegerField(default=0)),
                ('ships_lost', models.IntegerField(default=0)),
                ('solo_destroyed', models.IntegerField(default=0)),
                ('solo_lost', models.IntegerField(default=0)),
                ('active_pvp_kills', models.IntegerField(default=0)),
                ('last_kill', models.DateTimeField(default=None, null=True)),
                ('last_update', models.DateTimeField(default=datetime.datetime(2019, 12, 25, 10, 38, 4, 421195))),
                ('zk_12m', models.IntegerField(default=0)),
                ('zk_6m', models.IntegerField(default=0)),
                ('zk_3m', models.IntegerField(default=0)),
                ('character', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='zkill', to='eveonline.evecharacter')),
            ],
        ),
        migrations.CreateModel(
            name='zKillMonth',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField(default=0)),
                ('month', models.IntegerField(default=0)),
                ('ships_destroyed', models.IntegerField(default=0)),
                ('ships_lost', models.IntegerField(default=0)),
                ('isk_destroyed', models.BigIntegerField(default=0)),
                ('isk_lost', models.BigIntegerField(default=0)),
                ('last_update', models.DateTimeField(default=datetime.datetime(2019, 12, 25, 10, 38, 4, 421869))),
                ('char', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='aastatistics.statscharacter')),
            ],
        ),
    ]



from unittest import mock
from django.db.models.query_utils import Q
from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership
from django.contrib.auth.models import User
from django.utils import timezone
from datetime import timedelta
from . import filters
from .models import zKillStatsFilter
from aastatistics import models as a_models
from django.contrib.auth.models import User, Group

from aastatistics import models


class TestGroupBotFilters(TestCase):
    @classmethod
    def setUpTestData(cls):

        EveCharacter.objects.all().delete()
        User.objects.all().delete()
        CharacterOwnership.objects.all().delete()
        a_models.StatsCharacter.objects.all().delete()
        a_models.zKillMonth.objects.all().delete()
        
        userids = range(1,11)
        
        users = []
        characters = []
        for uid in userids:
            user = AuthUtils.create_user(f"User_{uid}")
            main_char = AuthUtils.add_main_character_2(user,
                                                       f"Main {uid}",
                                                       uid,
                                                       corp_id=1,
                                                       corp_name='Test Corp 1',
                                                       corp_ticker='TST1')
            CharacterOwnership.objects.create(user=user, character=main_char, owner_hash=f"main{uid}")

            characters.append(main_char)                                      
            users.append(user)

        # add some extra characters to users in 2 corps/alliance
        for uid in range(0,5): # test corp 2
            character = EveCharacter.objects.create(character_name=f'Alt {uid}', 
                                                    character_id=11+uid, 
                                                    corporation_name='Test Corp 2', 
                                                    corporation_id=2, 
                                                    corporation_ticker='TST2')
            CharacterOwnership.objects.create(character=character, 
                                              user=users[uid], 
                                              owner_hash=f'ownalt{11+uid}')
            characters.append(character)

        for uid in range(5,10): # Test alliance 1
            character = EveCharacter.objects.create(character_name=f'Alt {uid}', 
                                                    character_id=11+uid, 
                                                    corporation_name='Test Corp 3', 
                                                    corporation_id=3, 
                                                    corporation_ticker='TST3',
                                                    alliance_id=1,
                                                    alliance_name="Test Alliance 1",
                                                    alliance_ticker="TSTA1")
            CharacterOwnership.objects.create(character=character, 
                                              user=users[uid], 
                                              owner_hash=f'ownalt{11+uid}')
            characters.append(character)

        zkc = a_models.StatsCharacter.objects.create(character=characters[2],
                                                  zk_12m=500, zk_6m=250, zk_3m=100)
        zkc2 = a_models.StatsCharacter.objects.create(character=characters[4],
                                                  zk_12m=0, zk_6m=0, zk_3m=0)

        date_str= str(timezone.now().year)+str(timezone.now().month).zfill(2)
        a_models.zKillMonth.objects.create(char=zkc2, ships_destroyed=5, date_str=date_str, month=timezone.now().month, year=timezone.now().year)

    def test_user_zkill_pre_calc_12(self):
        users = {}
        for user in User.objects.all():
            users[user.pk] = None

        tests = {}
        for k,u in users.items():
            tests[k] = filters.check_kills_in_account(User.objects.get(pk=k), 12, 499)
        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_zkill_pre_calc_6(self):
        users = {}
        for user in User.objects.all():
            users[user.pk] = None
                    
        tests = {}
        for k,u in users.items():
            tests[k] = filters.check_kills_in_account(User.objects.get(pk=k), 6, 249)
            
        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_zkill_pre_calc_3(self):
        users = {}
        for user in User.objects.all():
            users[user.pk] = None
                    
        tests = {}
        for k,u in users.items():
            tests[k] = filters.check_kills_in_account(User.objects.get(pk=k), 3, 99)
            
        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertTrue(tests[3])
        self.assertFalse(tests[4])
        self.assertFalse(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_zkill_calc(self):
        users = {}
        for user in User.objects.all():
            users[user.pk] = None
                    
        tests = {}
        for k,u in users.items():
            tests[k] = filters.check_kills_in_account(User.objects.get(pk=k), 2, 4)
            
        self.assertFalse(tests[1])
        self.assertFalse(tests[2])
        self.assertFalse(tests[3])
        self.assertFalse(tests[4])
        self.assertTrue(tests[5])
        self.assertFalse(tests[6])
        self.assertFalse(tests[7])
        self.assertFalse(tests[8])
        self.assertFalse(tests[9])
        self.assertFalse(tests[10])

    def test_user_zkill_audit(self):
        users = []

        for user in User.objects.all():
            users.append(user.pk)

        s_filter = zKillStatsFilter.objects.create(name="3/99",description="3/99",
                                                    months=2, kill_count=4)

        tests = s_filter.audit_filter(User.objects.filter(id__in=users))

        self.assertFalse(tests[1]['check'])
        self.assertFalse(tests[2]['check'])
        self.assertFalse(tests[3]['check'])
        self.assertFalse(tests[4]['check'])
        self.assertTrue(tests[5]['check'])
        self.assertFalse(tests[6]['check'])
        self.assertFalse(tests[7]['check'])
        self.assertFalse(tests[8]['check'])
        self.assertFalse(tests[9]['check'])
        self.assertFalse(tests[10]['check'])



# Generated by Django 3.2.5 on 2021-07-15 10:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aastatistics', '0004_add_date_strs'),
    ]

    operations = [
        migrations.AlterField(
            model_name='zkillmonth',
            name='date_str',
            field=models.CharField(max_length=6),
        ),
    ]



from django.contrib import admin

from .models import StatsCharacter, zKillMonth, zKillStatsFilter

admin.site.register(StatsCharacter)


class zKillMonthAdmin(admin.ModelAdmin):
    list_display = ('char', 'year', 'month', 'last_update')
    search_fields = ['char__character__character_name']
    ordering = ('char', '-year', 'month')


class zKillStatsFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'kill_count', 'months']


admin.site.register(zKillStatsFilter, zKillStatsFilterAdmin)
admin.site.register(zKillMonth, zKillMonthAdmin)



import logging
import requests
import datetime
import csv

from celery import shared_task, chain

from .models import StatsCharacter, zKillMonth
from allianceauth.eveonline.models import EveCharacter
from django.db.models import Sum
from django.db.models.functions import Coalesce
from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from dateutil.relativedelta import relativedelta
from time import sleep
from math import floor
from allianceauth.services.tasks import QueueOnce
from . import app_settings

logger = logging.getLogger(__name__)

if apps.is_installed("allianceauth.corputils"):
    from allianceauth.corputils.models import CorpStats
elif apps.is_installed("corpstats"):
    from corpstats.models import CorpStat as CorpStats
else:
    pass


def update_character_stats(character_id):
    #logger.info('update_character_stats for %s starting' % str(character_id))
    # https://zkillboard.com/api/stats/characterID/####/
    _stats_request = requests.get("https://zkillboard.com/api/stats/characterID/" + str(character_id) + "/")
    _stats_json = _stats_request.json()
    sleep(1)

    _last_kill_date = None

    char_model, created = StatsCharacter.objects.get_or_create(character=EveCharacter.objects.get(character_id=int(character_id)))

    if len(_stats_json.get('months', [])) > 0:
        current_data = zKillMonth.objects.filter(char=char_model)
        years = {}

        for m in current_data:
            if m.year not in years:
                years[m.year]={}
            years[m.year][m.month]=m
        
        updates = []
        creates = []
        for key, month in _stats_json.get('months', []).items(): 
            new_model = False
            try:
                zkill_month = years[month.get('year')][month.get('month')]
            except KeyError as e:
                zkill_month = zKillMonth(char=char_model, year=month.get('year', 0), month=month.get('month', 0))
                new_model = True
            zkill_month.date_str = str(month.get('year'))+str(month.get('month')).zfill(2)
            zkill_month.ships_destroyed = month.get('shipsDestroyed', 0)
            zkill_month.ships_lost = month.get('shipsLost', 0)
            zkill_month.isk_destroyed = month.get('iskDestroyed', 0)
            zkill_month.isk_lost = month.get('iskLost', 0)
            zkill_month.last_update = datetime.datetime.utcnow().replace(tzinfo=timezone.utc)
            if new_model:
                creates.append(zkill_month)
            else:
                updates.append(zkill_month)
        
        if len(updates) > 0:
            zKillMonth.objects.bulk_update(updates, batch_size=500, fields=['ships_destroyed', 'ships_lost', 'isk_destroyed', 'isk_lost', 'last_update', 'date_str'])
        if len(creates) > 0:
            zKillMonth.objects.bulk_create(creates, batch_size=500)

    char_model.isk_destroyed = _stats_json.get('iskDestroyed', 0)
    char_model.isk_lost = _stats_json.get('iskLost', 0)
    char_model.all_time_sum = _stats_json.get('allTimeSum', 0)
    char_model.gang_ratio = _stats_json.get('gangRatio', 0)
    char_model.ships_destroyed = _stats_json.get('shipsDestroyed', 0)
    char_model.ships_lost = _stats_json.get('shipsLost', 0)
    char_model.solo_destroyed = _stats_json.get('soloDestroyed', 0)
    char_model.solo_lost = _stats_json.get('soloLost', 0)
    char_model.active_pvp_kills = _stats_json.get('activepvp', {}).get('kills', {}).get('count', 0)
    char_model.last_kill = _last_kill_date
    char_model.last_update = datetime.datetime.utcnow().replace(tzinfo=timezone.utc)
    char_model.save()

    #logger.info('update_character_stats for %s complete' % str(character_id))

@shared_task(bind=True, base=QueueOnce)
def update_char(self, char_id):
    try:
        now = datetime.datetime.now()
        skip_time =  timezone.now() - relativedelta(hours=24)
        dt12 = now - relativedelta(months=12)
        dt6 = now - relativedelta(months=6)
        dt3 = now - relativedelta(months=3)
        try:
            #logger.info('update_character_stats for %s starting' % str(char_id))
            update_character_stats(char_id)
        except:
            logger.error('update_character_stats failed for %s' % str(char_id))
            logging.exception("Messsage")
            sleep(1)  # had an error printed it and skipped it YOLO. better wait a sec to not overload the api
            return 0 # fail
        try:
            character = StatsCharacter.objects.get(character__character_id=char_id)
            qs = zKillMonth.objects.filter(char=character)
            qs_12m = qs.filter(year=dt12.year, month__gte=dt12.month) | \
                        qs.filter(year=now.year)
            qs_12m = qs_12m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
            qs_6m = qs.filter(year=dt6.year, month__gte=dt6.month)
            if now.month <= 6:
                qs_6m = qs_6m | qs.filter(year=now.year)
            qs_6m = qs_6m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
            qs_3m = qs.filter(year__gte=dt3.year, month__gte=dt3.month)
            if now.month <= 3:
                qs_3m = qs_3m | qs.filter(year=now.year)
            qs_3m = qs_3m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
            character.zk_12m = qs_12m
            character.zk_6m = qs_6m
            character.zk_3m = qs_3m
            character.save()
        except ObjectDoesNotExist:
            pass # broken

    except Exception as e:
        logger.error("failed to update {}: {}".format(char_id, e))
        return 0  # fail
    
    return 1  # pass


@shared_task()
def run_stat_model_update():
    # update all corpstat'd characters
    #logger.info('start')
    member_alliances = app_settings.MEMBER_ALLIANCES
    linked_chars = EveCharacter.objects.filter(alliance_id__in=member_alliances)  # get all authenticated characters in corp from auth internals

    sig_list = []

    for alt in linked_chars:
        if alt.alliance_id in member_alliances:
            sig_list.append(update_char.si(alt.character_id))

    chain(sig_list).apply_async(priority=8)


@shared_task()
def run_aggregate_update():
    # update all corpstat'd characters
    #logger.info('start')
    active_corp_stats = CorpStats.objects.all()
    member_alliances = app_settings.MEMBER_ALLIANCES
    now = datetime.datetime.now()
    dt12 = now - relativedelta(months=12)
    dt6 = now - relativedelta(months=6)
    dt3 = now - relativedelta(months=3)
    for cs in active_corp_stats:
        members = cs.mains
        for member in members:
            for alt in member.alts:
                if alt.alliance_id in member_alliances:
                    try:
                        logger.info('update_character_agregates for %s starting' % str(alt.character_name))
                        character = StatsCharacter.objects.get(character__character_id=alt.character_id)
                        qs = zKillMonth.objects.filter(char=character)
                        qs_12m = qs.filter(year=dt12.year, month__gte=dt12.month) | \
                                 qs.filter(year=now.year)
                        qs_12m = qs_12m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
                        qs_6m = qs.filter(year=dt6.year, month__gte=dt6.month)
                        if now.month <= 6:
                            qs_6m = qs_6m | qs.filter(year=now.year)
                        qs_6m = qs_6m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
                        qs_3m = qs.filter(year__gte=dt3.year, month__gte=dt3.month)
                        if now.month <= 3:
                            qs_3m = qs_3m | qs.filter(year=now.year)
                        qs_3m = qs_3m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)
                        character.zk_12m = qs_12m
                        character.zk_6m = qs_6m
                        character.zk_3m = qs_3m
                        character.save()
                    except ObjectDoesNotExist:
                        logger.info('failed update_character_agregates for %s starting' % str(alt.character_name))
                        pass # who knows    

def output_stats(file_output=True):
    active_corp_stats = CorpStats.objects.all()

    out_arr={}
    for cs in active_corp_stats:
        members = cs.mains
        for member in members:
            print("Adding: %s" % member.character.character_name)
            now = datetime.datetime.now()
            #try:
            in_char = EveCharacter.objects.get(
                character_id=member.character.character_id).character_ownership.user.profile.main_character
            character_list = in_char.character_ownership.user.character_ownerships.all().select_related('character')
            character_ids = set(character_list.values_list('character__character_id', flat=True))

            month_12_ago = ((now.month - 1 - 12) % 12 + 1)
            month_6_ago = ((now.month - 1 - 6) % 12 + 1)
            month_3_ago = ((now.month - 1 - 3) % 12 + 1)
            year_12_ago = (now.year + floor((now.month - 12) / 12))
            year_6_ago = (now.year + floor((now.month - 6) / 12))
            year_3_ago = (now.year + floor((now.month - 3) / 12))

            character = StatsCharacter.objects.filter(character__character_id__in=character_ids)

            qs = zKillMonth.objects.filter(char__in=character)

            qs_12m = qs.filter(year=year_12_ago, month__gte=month_12_ago) | \
                     qs.filter(year=now.year)
            qs_12m = qs_12m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get(
                'ship_destroyed_sum', 0)

            qs_6m = qs.filter(year=year_6_ago, month__gte=month_6_ago)
            if now.month < 6:
                qs_6m = qs_6m | qs.filter(year=now.year)
            qs_6m = qs_6m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get(
                'ship_destroyed_sum', 0)

            qs_3m = qs.filter(year__gte=year_3_ago, month__gte=month_3_ago)
            if now.month < 3:
                qs_3m = qs_3m | qs.filter(year=now.year)
            qs_3m = qs_3m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get(
                'ship_destroyed_sum', 0)

            out_str=[]
            out_str.append(in_char.character_name)
            out_str.append(in_char.corporation_name)
            out_str.append(str(qs_12m))
            out_str.append(str(qs_6m))
            out_str.append(str(qs_3m))
            out_arr[in_char.character_name]=out_str
            #except:
             #   pass
    
    if file_output:
        with open('auth_zkill_dump.csv', 'w') as writeFile:
            writer = csv.writer(writeFile)
            writer.writerow(['Name', 'Corp', '12m', '6m', '3m'])
            for char, data in out_arr.items():
                writer.writerow(data)
        writeFile.close()
    else:
        return out_arr
    



import csv
from django.http import HttpResponse
from .tasks import output_stats
from django.contrib.auth.decorators import login_required, permission_required


@login_required
@permission_required('corputils.view_alliance_corpstats')
def outputcsv(request):
    # Create the HttpResponse object with the appropriate CSV header.
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="auth_zkill_dump.csv"'

    writer = csv.writer(response)
    writer.writerow(['Name', 'Corp', '12m', '6m', '3m'])
    for char, data in output_stats(file_output=False).items():
        writer.writerow(data)

    return response



from django.conf.urls import url

import allianceauth.urls
from . import views

urlpatterns = allianceauth.urls.urlpatterns

urlpatterns += [
    # Navhelper test urls
    url(r"^main-page/$", views.page, name="p1"),
    url(r"^main-page/sub-section/$", views.page, name="p1-s1"),
    url(r"^second-page/$", views.page, name="p1"),
]



from django import template
from django.db.models import Sum
from django.db.models.functions import Coalesce
from math import floor

import datetime

from aastatistics.models import StatsCharacter, zKillMonth
from allianceauth.eveonline.models import EveCharacter

register = template.Library()


@register.filter(name='get_ytd_kills_single')
def get_ytd_kills_single(input_id, month=None):
    try:
        now = datetime.datetime.now()

        month_12_ago = ((now.month - 1 - 12) % 12 + 1)
        month_6_ago = ((now.month - 1 - 6) % 12 + 1)
        month_3_ago = ((now.month - 1 - 3) % 12 + 1)
        year_12_ago = (now.year + floor((now.month - 12) / 12))
        year_6_ago = (now.year + floor((now.month - 6) / 12))
        year_3_ago = (now.year + floor((now.month - 3) / 12))

        character = StatsCharacter.objects.filter(character__character_id=input_id)

        qs = zKillMonth.objects.filter(char__in=character)

        qs_12m = qs.filter(year=year_12_ago, month__gte=month_12_ago) | \
                 qs.filter(year=now.year)
        qs_12m = qs_12m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        qs_6m = qs.filter(year=year_6_ago, month__gte=month_6_ago)
        if now.month < 6:
            qs_6m = qs_6m | qs.filter(year=now.year)
        qs_6m = qs_6m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        qs_3m = qs.filter(year__gte=year_3_ago, month__gte=month_3_ago)
        if now.month < 3:
            qs_3m = qs_3m | qs.filter(year=now.year)
        qs_3m = qs_3m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        return "<td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td>" % \
               (str(qs_12m if qs_12m else 0), str(qs_6m if qs_6m else 0), str(qs_3m if qs_3m else 0))

    except:
        return "<td class=\"text-center\">%s</td><td class=\"text-center\">%s</td><td class=\"text-center\">%s</td>" % (str(0), str(0), str(0))


@register.filter(name='get_ytd_kills_account')
def get_ytd_kills_account(input_id):
    try:
        now = datetime.datetime.now()

        in_char = EveCharacter.objects.get(character_id=input_id).character_ownership.user.profile.main_character
        character_list = in_char.character_ownership.user.character_ownerships.all().select_related('character')
        character_ids = set(character_list.values_list('character__character_id', flat=True))

        month_12_ago = ((now.month - 1 - 12) % 12 + 1)
        month_6_ago = ((now.month - 1 - 6) % 12 + 1)
        month_3_ago = ((now.month - 1 - 3) % 12 + 1)
        year_12_ago = (now.year + floor((now.month - 12) / 12))
        year_6_ago = (now.year + floor((now.month - 6) / 12))
        year_3_ago = (now.year + floor((now.month - 3) / 12))

        character = StatsCharacter.objects.filter(character__character_id__in=character_ids)

        qs = zKillMonth.objects.filter(char__in=character)

        qs_12m = qs.filter(year=year_12_ago, month__gte=month_12_ago) | \
                 qs.filter(year=now.year)
        qs_12m = qs_12m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        qs_6m = qs.filter(year=year_6_ago, month__gte=month_6_ago)
        if now.month<6:
            qs_6m = qs_6m | qs.filter(year=now.year)
        qs_6m = qs_6m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        qs_3m = qs.filter(year__gte=year_3_ago, month__gte=month_3_ago)
        if now.month < 3:
            qs_3m = qs_3m | qs.filter(year=now.year)
        qs_3m = qs_3m.aggregate(ship_destroyed_sum=Coalesce(Sum('ships_destroyed'), 0)).get('ship_destroyed_sum', 0)

        return "<td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td>" % \
               (str(qs_12m if qs_12m else 0), str(qs_6m if qs_6m else 0), str(qs_3m if qs_3m else 0))

    except:
        return "<td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td><td class=\"text-center\" style=\"vertical-align:middle\">%s</td>" % (str(0), str(0), str(0))



# Generated by Django 3.2.5 on 2021-07-15 10:31

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aastatistics', '0002_auto_20210106_0803'),
    ]

    operations = [
        migrations.AddField(
            model_name='zkillmonth',
            name='date_str',
            field=models.CharField(max_length=6, null=True),
        ),
        migrations.AlterField(
            model_name='statscharacter',
            name='last_update',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0)),
        ),
        migrations.AlterField(
            model_name='zkillmonth',
            name='last_update',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0)),
        ),
    ]
